@@ -47,10 +47,11 @@
 #include <string.h>
 #include <signal.h>
 
-#define STOPIT "/run/runit.stopit"
+#define STOPIT "/etc/runit/stopit"
 #define NOSYNC "/etc/runit/nosync"
 #define DOFSCK "/forcefsck"
 #define NOFSCK "/fastboot"
+#define WTMPDB "/usr/bin/wtmpdb"
 #define SUSAGE "Usage: shutdown [-F] [-f] [-r] [-h] [now] \n"
 #define HUSAGE "Usage: halt  [-n] [-f] [-w] \n"
 
@@ -79,6 +80,7 @@ struct config {
 	bool nosync;
 	bool nofsck;
 	bool dofsck;
+	 bool wtmp;
 	bool noop;		/*stop and exit 0 without actually reboot or poweroff  */
 	bool error;
 };
@@ -148,12 +150,21 @@ runit_shutdown(shutdown_action action)
 	return false;
 }
 
+int write_wtmpdb(void) {
+	char* args[] = {WTMPDB, "shutdown", NULL };
+	if (execv(WTMPDB, args) == -1) {
+		write1("halt: warning: failed to write a wtmpdb shutdown entry\n");
+		return 0;
+	}
+}
+
 int open_trunc(const char *fn)
 { return open(fn,O_WRONLY | O_NDELAY | O_TRUNC | O_CREAT,0644); }
 
 static void
 parse_command_line(struct config *cfg, int argc, char **argv)
 {
+	int options_ended = 0;
 	int i;
 
 	cfg->action    = ACTION_HALT;
@@ -161,6 +172,7 @@ parse_command_line(struct config *cfg, int argc, char **argv)
 	cfg->nosync = false;
 	cfg->dofsck = false;
 	cfg->nofsck = false;
+	cfg->wtmp = false;
 	cfg->noop = false;
 	cfg->error = false;
 
@@ -186,22 +198,30 @@ parse_command_line(struct config *cfg, int argc, char **argv)
 				cfg->dofsck = true;
 				continue;
 			}
+			if (strcmp(argv[i], "-P") == 0 ||
+				strcmp(argv[i], "-H") == 0) {
+				/* Halt actions not distinguished */
+				continue;
+			}
 			if (argv[i][0]  == '-') {
 				write2(SUSAGE);
 				cfg->error = true;
 				break;
 			}
 			if (strcmp(argv[i], "now") == 0) {
+				options_ended = 1;
 				/*write1("time argument ignored\n"); */
 				continue;
 			}
-			else {
-				/* time spec and shutdown message not supported */
+			else if (!options_ended) {
+				/* time spec not supported */
 				write1("time argument: not supported\n");
-				write1("shutdown message: not supported\n");
 				write2(SUSAGE);
 				cfg->error = true;
 				break;
+			} else {
+				write1("shutdown message: ignored\n");
+				break;
 			}
 		}
 		else {
@@ -220,12 +240,11 @@ parse_command_line(struct config *cfg, int argc, char **argv)
 				continue;
 			}
 			if (strcmp(argv[i], "-w") == 0) {
-				write1("WARNING: writing wtmp with -w is not supported for now\n");
-				cfg->noop = true;
+				cfg->wtmp =true;
 				continue;
 			}
 			if (strcmp(argv[i], "--wtmp-only") == 0) {
-				cfg->noop = true;
+				cfg->wtmp =true;
 				continue;
 			}
 			if (strcmp(argv[i], "-n") == 0) {
@@ -262,6 +281,10 @@ main(int argc, char **argv)
 		// No-Op: see #919699 and #899246
 		return 0;
 	}
+	if (cfg.wtmp) {
+		write_wtmpdb();
+		return 0;
+	}
 	if (cfg.dofsck) {
 		if (close(open_trunc(DOFSCK) == -1)) {
 			write2("Failed to write forcefsck file.\n");
